# Imports Contents from a specific repo as reminders.

import requests

from .reminders import reminder
from .utilities import parser


def gitContents(prompt: list[str]) -> tuple[bool, list[reminder]]:
    """
    Returns a list of contents from a GitHub repo as reminders.
    """

    _, sdOpts, ddOpts = parser(prompt)

    try:
        assert "r" in sdOpts
        assert isinstance(sdOpts["r"], str)
        assert "/" in sdOpts["r"] and len(sdOpts["r"]) >= 3

        if "u" in sdOpts:
            assert isinstance(sdOpts["u"], str)
            assert len(sdOpts["u"])

        repo = sdOpts["r"]
        content = "pulls" if "pulls" in ddOpts else "issues"

        url: str = "https://api.github.com/repos/{}/{}"
        url = url.format(repo, content)

        contents: list = requests.get(url).json()

        # contents might be of type 'dict' on errors.
        assert isinstance(contents, list)

    except:  # This should be avoided.
        return False, []

    reminders: list[reminder] = []

    for content in contents:
        gitContent: dict[str, str] = {}
        gitContent["title"] = content["title"]
        gitContent["description"] = repo

        if "u" not in sdOpts:
            reminders.append(reminder(gitContent, True))

        # Checks assignee or requested reviewer on pulls.
        else:
            gitContent["description"] += ":" + sdOpts["u"]
            search = content["assignees"]

            if "pulls" in ddOpts:
                search += content["requested_reviewers"]

            if sdOpts["u"] in [user["login"] for user in search]:
                reminders.append(reminder(gitContent, True))

    return True, reminders



# Handles promemoria files.

import pickle
from os import makedirs
from os.path import expanduser

from .reminders import reminder


def getReminders() -> list[reminder]:
    """
    Returns reminders from ~/.promemoria/reminders.pickle
    """

    # Creates or checks reminders' folder.
    global path
    path = expanduser("~") + "/.promemoria/"
    makedirs(path, exist_ok=True)

    # Loads reminders.
    try:
        file = open(path + "reminders.pickle", "rb")

        reminders: list[reminder] = pickle.load(file)
        file.close()

        for rem in reminders:
            if not isinstance(rem, reminder):
                # Raises this error to recreate the file.
                raise FileNotFoundError

    except FileNotFoundError:
        reminders: list[reminder] = []

        saveReminders(reminders)

    except:
        reminders = []

    # Runs a check on reminders.
    for rem in reminders:
        rem.check()

    return reminders


def saveReminders(reminders: list[reminder]) -> None:
    """
    Saves reminders to ~/.promemoria/reminders.pickle
    """

    file = open(path + "reminders.pickle", "wb")
    pickle.dump(reminders, file)
    file.close()



from __future__ import annotations

from datetime import datetime

from colorama import Fore, Style

from .utilities import parser, valiDate, msg


class reminder:
    def __init__(self: reminder, prompt: "list, dict", git: bool = False):
        if git:
            # Create a reminder from a git issue or pull request.
            # prompt: dict[str, str]

            self.title = prompt["title"]
            self.description = prompt["description"]

            self.priority = 2  # Default priority.
            self.date = ""
            self.time = ""

            # Flags.
            self.expired: bool = False
            self.dismissed: bool = False

        else:
            try:
                # Create a reminder 'from scratch'.
                # prompt: list[str]

                _, sdOpts, _ = parser(prompt)

                # Checks title.
                assert "t" in sdOpts

                # Title.
                self.title: str = sdOpts["t"]

                # Checks priority.
                if "p" in sdOpts:
                    assert isinstance(sdOpts["p"], int)
                    assert 1 <= sdOpts["p"] <= 3

                    # Priority.
                    self.priority: int = sdOpts["p"]

                else:
                    self.priority = 0

                # Checks description.
                if "de" in sdOpts:
                    assert isinstance(sdOpts["de"], str)

                    # Description.
                    self.description: str = sdOpts["de"]

                else:
                    self.description = ""

                # Checks date.
                if "da" in sdOpts:
                    assert isinstance(sdOpts["da"], str)
                    assert valiDate(sdOpts["da"])

                    # Date.
                    self.date: str = sdOpts["da"]

                else:
                    self.date = ""

                # Checks time.
                if "ti" in sdOpts:
                    assert isinstance(sdOpts["ti"], str)
                    assert valiDate(sdOpts["ti"], "%H:%M")

                    if not self.date:
                        self.date = datetime.now().strftime("%Y-%m-%d")

                    # Time.
                    self.time: str = sdOpts["ti"]

                else:
                    self.time = ""

                # Flags.
                self.expired: bool = False
                self.dismissed: bool = False
                self.confirmation: bool = True

                # Post-creation check.
                self.check()

                msg("Reminder created succesfully.")

            except AssertionError:
                msg("Reminder creation failed.", error=True)
                self.confirmation: bool = False

    def __str__(self: reminder, index: int = -1) -> str:
        # Mark.
        mark = "\u25cf " if self.dismissed else "\u25ef "

        if index != -1:
            mark += "[{}] ".format(index)

        # Lenght of 'mark' in spaces.
        spaces = len(mark) * " "

        # Title.
        # Striked on dismissed reminders.
        if self.dismissed:
            title = Style.DIM + self.title + Style.RESET_ALL

        else:
            title = Style.BRIGHT + self.title + Style.RESET_ALL

        string = mark + title

        # Priority.
        if self.priority:
            priority = " " + Fore.RED + "!" * self.priority + Fore.RESET

            string += priority

        # Description.
        if self.description:
            description = Style.DIM + self.description + Style.RESET_ALL

            string += "\n" + spaces + description

        # Date.
        if self.date or self.time:
            date: str = Fore.GREEN if not self.expired else Fore.RED
            date += self.date + "{}{}" + Fore.RESET
            date = date.format(" " if self.date else "", self.time)

            string += "\n" + spaces + date

        # Finally returns string.
        return string

    def check(self: reminder) -> None:
        if self.date:
            date = datetime.strptime(self.date, "%Y-%m-%d")

            if self.time:
                time = datetime.strptime(self.time, "%H:%M")

            now = datetime.now()

            # Checks expiration.
            if date < now:
                if self.time:
                    if time.hour <= now.hour and time.minute < now.minute:
                        self.expired = True

                    else:
                        self.expired = False

                else:
                    self.expired = True

            else:
                self.expired = False

    def toggle(self: reminder) -> None:
        """
        Toggles itself.
        """

        self.dismissed = not self.dismissed



# promemoria's help

from colorama import Fore, Style


def help() -> str:
    """
    Returns the help string.
    """
    spaces = " " * 4
    introduction: str = "Base commands."
    introduction += "\n" + "-" * len(introduction)

    # Empty command.
    cmd: str = Style.BRIGHT + "promemoria " + Style.RESET_ALL
    cmd += Style.DIM + "Shows the list of active reminders." + Style.RESET_ALL

    cmd += "\n" + spaces + Style.BRIGHT + "--all " + Style.RESET_ALL
    cmd += Style.DIM + "Shows every reminder." + Style.RESET_ALL

    # new.
    cmdNew: str = Style.BRIGHT + "promemoria new " + Style.RESET_ALL
    cmdNew += Style.DIM + "Creates a new reminder" + Style.RESET_ALL

    cmdNew += "\n" + spaces + Fore.RED + Style.BRIGHT + "-t " + Style.RESET_ALL
    cmdNew += Style.DIM + "title, string." + Style.RESET_ALL

    cmdNew += "\n" + spaces + Style.BRIGHT + "-de " + Style.RESET_ALL
    cmdNew += Style.DIM + "description, string. " + Style.RESET_ALL

    cmdNew += "\n" + spaces + Style.BRIGHT + "-da " + Style.RESET_ALL
    cmdNew += Style.DIM + "date, string, ISO 8601 compliant." + Style.RESET_ALL

    cmdNew += "\n" + spaces + Style.BRIGHT + "-ti " + Style.RESET_ALL
    cmdNew += Style.DIM + "time, string." + Style.RESET_ALL

    cmdNew += "\n" + spaces + Style.BRIGHT + "-p " + Style.RESET_ALL
    cmdNew += Style.DIM + "priority, integer, [1-3]." + Style.RESET_ALL

    # delete.
    cmdDelete: str = Style.BRIGHT + "promemoria delete " + Style.RESET_ALL
    cmdDelete += Style.DIM + "Deletes the specified reminder" + Style.RESET_ALL

    cmdDelete += "\n" + spaces + Fore.RED + Style.BRIGHT + "-i " + Style.RESET_ALL
    cmdDelete += Style.DIM + "index, integer." + Style.RESET_ALL

    # toggle.
    cmdToggle: str = Style.BRIGHT + "promemoria toggle " + Style.RESET_ALL
    cmdToggle += Style.DIM + "Toggles the specified reminder" + Style.RESET_ALL

    cmdToggle += "\n" + spaces + Fore.RED + Style.BRIGHT + "-i " + Style.RESET_ALL
    cmdToggle += Style.DIM + "index, integer." + Style.RESET_ALL

    # clear.
    cmdClear: str = Style.BRIGHT + "promemoria clear " + Style.RESET_ALL
    cmdClear += Style.DIM + "Deletes every reminder" + Style.RESET_ALL

    # GitHub integration.
    introductionGit: str = "GitHub integration."
    introductionGit += "\n" + "-" * len(introductionGit)

    # git.
    cmdGit: str = Style.BRIGHT + "promemoria gh " + Style.RESET_ALL
    cmdGit += (
        Style.DIM + "Imports issues and pull requests from GitHub" + Style.RESET_ALL
    )

    cmdGit += "\n" + spaces + Fore.RED + Style.BRIGHT + "-r " + Style.RESET_ALL
    cmdGit += Style.DIM + "public repo, string [user/repo]." + Style.RESET_ALL

    cmdGit += "\n" + spaces + Style.BRIGHT + "-u " + Style.RESET_ALL
    cmdGit += Style.DIM + "user, string." + Style.RESET_ALL

    cmdGit += "\n" + spaces + Style.BRIGHT + "--pulls " + Style.RESET_ALL
    cmdGit += Style.DIM + "Imports pull requests instead of issues." + Style.RESET_ALL

    # Full help.
    entries: list[str] = [introduction, cmd, cmdNew, cmdDelete, cmdToggle, cmdClear]

    # GitHub integration.
    entries += [introductionGit, cmdGit]

    return "\n\n".join(entries)
